// -*- mode: C++; c-file-style: "mnemonic"; -*-

/*
  Various number classes.

  Copyright 1999 Kasper Peeters.

  Please send bugs, changes, problems to bugs@mnemonic.browser.org

  This program and its components are free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2, or (at your option) any later version.
 
  This program and its components are distributed in the hope that it
  will be useful, but WITHOUT ANY WARRANTY; without even the implied
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License in
  the file COPYING accompanying this program; if not, write to the
  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef numbers_hh_
#define numbers_hh_

#include <string>
#include <stdint.h>
#include <iostream.h>

namespace layout {

 static const float   PERCENTAGE_MULTIPLIER=100000.0;
 static const int32_t EPSILON=1;
 static const int32_t INFINITY=(1L<<20); // everything equal or bigger than this is infinity

 class fixword {
	 public:
		 fixword(void);
		 fixword(const int32_t&);
		 fixword& operator=(const int32_t&);
		 friend ostream& operator<<(ostream&, const fixword&);
		 int32_t  raw;
		 int32_t  numer(void) const;
		 int32_t  denom(void) const;
		 double approx(void) const;
 };
 
 class point {
	 public:
		 point(void);
		 point(int32_t,int32_t);
		 
		 int32_t x,y;
		 point& operator+=(const point&);
 };

 class rectangle {
	 public:
		 rectangle(void);
		 rectangle(point, point);
		 
		 point top_left;
		 point size;
 };

 point operator+(const point&, const point&);

 static const float dots_per_inch=95.0;

 class length {
	 public:
		 enum length_unit { px, pt, pc, in, bp, cm, mm, dd, cc, sp };
		 length(void);
		 length(const std::string&);
		 length(float, length_unit);
		 
		 inline float       number(void) const;
		 inline length_unit unit(void) const;

		 int32_t     pixels() const;
		 length      convert(length_unit) const;
	 protected:
		 float       num_;
		 length_unit unit_;
		 float scaled_points_(void) const;
 };

}; 

ostream& operator<<(ostream&, const layout::point&);
ostream& operator<<(ostream&, const layout::length&);


#endif 
